// ignore_for_file: depend_on_referenced_packages

import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import 'package:tencent_cloud_chat/components/component_config/tencent_cloud_chat_message_common_defines.dart';
import 'package:tencent_cloud_chat/tencent_cloud_chat.dart';
import 'package:tencent_cloud_chat/tuicore/tencent_cloud_chat_core.dart';
import 'package:tencent_cloud_chat_common/base/tencent_cloud_chat_state_widget.dart';
import 'package:tencent_cloud_chat_common/widgets/group_member_selector/tencent_cloud_chat_group_member_selector.dart';
import 'package:tencent_cloud_chat_common/widgets/modal/bottom_modal.dart';
import 'package:tencent_cloud_chat_message/data/tencent_cloud_chat_message_separate_data.dart';
import 'package:tencent_cloud_chat_message/data/tencent_cloud_chat_message_separate_data_notifier.dart';
import 'package:tencent_cloud_chat_message/tencent_cloud_chat_message_builders.dart';

class TencentCloudChatMessageInputContainer extends StatefulWidget {
  final String? userID;
  final String? groupID;

  const TencentCloudChatMessageInputContainer({super.key, this.userID, this.groupID});

  @override
  State<TencentCloudChatMessageInputContainer> createState() => _TencentCloudChatMessageInputContainerState();
}

class _TencentCloudChatMessageInputContainerState extends TencentCloudChatState<TencentCloudChatMessageInputContainer> {
  late TencentCloudChatMessageSeparateDataProvider dataProvider;
  bool _inSelectMode = false;
  V2TimMessage? _repliedMessage;
  List<TencentCloudChatMessageGeneralOptionItem> _attachmentOptions = [];

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    dataProvider = TencentCloudChatMessageDataProviderInherited.of(context);
    dataProvider.addListener(_dataProviderListener);
    _dataProviderListener();
    _buildAttachmentOptionsList();
  }

  void _buildAttachmentOptionsList() {
    final config = dataProvider.config;
    final defaultOptions = [
      TencentCloudChatMessageGeneralOptionItem(icon: Icons.insert_drive_file, label: tL10n.file, onTap: _sendFileFromExplorer),
      TencentCloudChatMessageGeneralOptionItem(icon: Icons.photo_library, label: tL10n.album, onTap: _sendMediaFromGallery),
      TencentCloudChatMessageGeneralOptionItem(icon: Icons.location_on, label: tL10n.location, onTap: () {}),
      TencentCloudChatMessageGeneralOptionItem(icon: Icons.call, label: tL10n.call, onTap: _startCall),
    ];
    final additionalAttachmentOptions = config.additionalAttachmentOptions(
      userID: dataProvider.userID,
      groupID: dataProvider.groupID,
    );
    _attachmentOptions = [...defaultOptions, ...additionalAttachmentOptions];
  }

  void _startCall() {
    showTencentCloudChatBottomModal(
      context: context,
      actions: [
        TencentCloudChatModalAction(
          label: tL10n.voiceCall,
          icon: Icons.call,
          onTap: () {
            if (widget.userID != null) {
              TencentCloudChatTUICore.audioCall(userids: [widget.userID!]);
            }
          },
        ),
        TencentCloudChatModalAction(
          label: tL10n.videoCall,
          icon: Icons.videocam,
          onTap: () {
            if (widget.userID != null) {
              TencentCloudChatTUICore.videoCall(userids: [widget.userID!]);
            }
          },
        ),
      ],
    );
  }

  void _sendMediaFromGallery() async {
    final ImagePicker picker = ImagePicker();
    final List<XFile> files = await picker.pickMultipleMedia();

    for (final file in files) {
      final String fileExtension = file.path.split('.').last.toLowerCase();
      if (['jpg', 'jpeg', 'png', 'gif', 'bmp', 'tiff', 'webp'].contains(fileExtension)) {
        _sendImageMessage(imagePath: file.path);
      } else if (['mp4', 'mov', 'avi', 'mkv', 'flv', 'wmv', 'mpeg', 'webm', '3gp'].contains(fileExtension)) {
        _sendVideoMessage(videoPath: file.path);
      } else {
        // Unsupported file type
        debugPrint('Unsupported file type: $fileExtension');
      }
    }
  }

  void _sendFileFromExplorer() async {
    FilePickerResult? result = await FilePicker.platform.pickFiles(allowMultiple: true);

    if (result != null) {
      List<File> files = result.paths.map((path) => File(path!)).toList();
      for (var element in files) {
        _sendFileMessage(filePath: element.path);
      }
    }
  }

  @override
  void dispose() {
    super.dispose();
    dataProvider.removeListener(_dataProviderListener);
  }

  void _dataProviderListener() {
    /// _inSelectMode
    if (_inSelectMode != dataProvider.inSelectMode) {
      setState(() {
        _inSelectMode = dataProvider.inSelectMode;
      });
    }

    /// _repliedMessage
    if (_repliedMessage != dataProvider.repliedMessage) {
      setState(() {
        _repliedMessage = dataProvider.repliedMessage;
      });
    }
  }

  void _sendTextMessage({
    required String text,
    List<String>? mentionedUsers,
  }) {
    dataProvider.sendTextMessage(text, mentionedUsers ?? []);
  }

  void _sendImageMessage({required String imagePath}) {
    dataProvider.sendImageMessage(imagePath);
  }

  void _sendVideoMessage({required String videoPath}) {
    dataProvider.sendVideoMessage(videoPath);
  }

  void _sendFileMessage({required String filePath}) {
    dataProvider.sendFileMessage(filePath);
  }

  void _sendVoiceMessage({required String voicePath, required int duration}) {
    dataProvider.sendVoiceMessage(voicePath, duration);
  }

  Future<List<V2TimGroupMemberFullInfo>> _onChooseGroupMembers({int? maxSelectionAmount, String? onSelectLabel, String? title}) async {
    final List<V2TimGroupMemberFullInfo> memberList = await showGroupMemberSelector(
      groupMemberList: dataProvider.groupMemberList.where((element) => element != null).map((e) => e!).toList(),
      context: context,
      maxSelectionAmount: maxSelectionAmount,
      onSelectLabel: onSelectLabel,
      title: title,
    );
    return memberList;
  }

  @override
  Widget defaultBuilder(BuildContext context) {
    return TencentCloudChatMessageBuilders.getMessageInputBuilder(
      controller: dataProvider.messageController,
      sendTextMessage: _sendTextMessage,
      sendVideoMessage: _sendVideoMessage,
      sendImageMessage: _sendImageMessage,
      sendVoiceMessage: _sendVoiceMessage,
      sendFileMessage: _sendFileMessage,
      userID: widget.userID,
      groupID: widget.groupID,
      attachmentOptions: _attachmentOptions,
      inSelectMode: _inSelectMode,
      selectedMessages: TencentCloudChatMessageDataProviderInherited.of(context).selectedMessages,
      repliedMessage: _repliedMessage,
      onChooseGroupMembers: _onChooseGroupMembers,
    );
  }
}
