import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:tencent_cloud_chat/components/component_config/tencent_cloud_chat_message_common_defines.dart';
import 'package:tencent_cloud_chat/components/component_config/tencent_cloud_chat_message_config.dart';
import 'package:tencent_cloud_chat/data/message/tencent_cloud_chat_message_data.dart';
import 'package:tencent_cloud_chat/tencent_cloud_chat.dart';
import 'package:tencent_cloud_chat/utils/tencent_cloud_chat_utils.dart';
import 'package:tencent_cloud_chat_common/base/tencent_cloud_chat_state_widget.dart';
import 'package:tencent_cloud_chat_message/data/tencent_cloud_chat_message_separate_data.dart';
import 'package:tencent_cloud_chat_message/data/tencent_cloud_chat_message_separate_data_notifier.dart';
import 'package:tencent_cloud_chat_message/tencent_cloud_chat_message_builders.dart';
import 'package:tencent_cloud_chat_message/tencent_cloud_chat_message_input/forward/tencent_cloud_chat_message_forward_container.dart';

class TencentCloudChatMessageItemWithMenuContainer extends StatefulWidget {
  final Widget Function({required bool renderOnMenuPreview}) messageItem;
  final bool useMessageReaction;
  final V2TimMessage message;
  final bool isMergeMessage;

  const TencentCloudChatMessageItemWithMenuContainer({
    super.key,
    required this.messageItem,
    required this.useMessageReaction,
    required this.message,
    required this.isMergeMessage,
  });

  @override
  State<TencentCloudChatMessageItemWithMenuContainer> createState() => _TencentCloudChatMessageItemWithMenuContainerState();
}

class _TencentCloudChatMessageItemWithMenuContainerState extends TencentCloudChatState<TencentCloudChatMessageItemWithMenuContainer> {
  late TencentCloudChatMessageSeparateDataProvider dataProvider;
  final Stream<TencentCloudChatMessageData<dynamic>>? _messageDataStream = TencentCloudChat.eventBusInstance.on<TencentCloudChatMessageData>();
  late StreamSubscription<TencentCloudChatMessageData<dynamic>>? _messageDataSubscription;

  List<TencentCloudChatMessageGeneralOptionItem> _menuOptions = [];
  int _menuCloser = 0;
  V2TimMessageReceipt? _messageReceipt;

  late V2TimMessage message;

  // This method handles changes in message data.
  void _messageDataHandler(TencentCloudChatMessageData messageData) {
    final msgID = message.msgID ?? "";
    final bool isGroup = TencentCloudChatUtils.checkString(message.groupID) != null;
    final TencentCloudChatMessageDataKeys messageDataKeys = messageData.currentUpdatedFields;

    switch (messageDataKeys) {
      case TencentCloudChatMessageDataKeys.messageHighlighted:
        break;
      case TencentCloudChatMessageDataKeys.none:
        break;
      case TencentCloudChatMessageDataKeys.messageReadReceipts:
        final receipt = messageData.getMessageReadReceipt(
          msgID: msgID,
          userID: message.userID ?? "",
          timestamp: message.timestamp ?? 0,
        );
        if (isGroup && (_messageReceipt == null || _messageReceipt?.readCount != receipt.readCount || _messageReceipt?.unreadCount != receipt.unreadCount)) {
          setState(() {
            _messageReceipt = receipt;
          });
          _generateMenuOptions(messageReceipt: receipt);
        }
      case TencentCloudChatMessageDataKeys.messageList:
        break;
      case TencentCloudChatMessageDataKeys.downloadMessage:
        break;
      case TencentCloudChatMessageDataKeys.sendMessageProgress:
        if (messageData.messageProgressData.containsKey(msgID)) {
          if (messageData.messageProgressData[msgID] != null) {
            setState(() {
              var data = messageData.messageProgressData[msgID];
              if (data != null) {
                message = data.message;
              }
            });
            _generateMenuOptions();
          }
        }
        break;
      case TencentCloudChatMessageDataKeys.currentPlayAudioInfo:
        break;
    }
  }

  @override
  void initState() {
    super.initState();
    message = widget.message;
    _messageDataSubscription = _messageDataStream?.listen(_messageDataHandler);
    _messageReceipt = TencentCloudChat.dataInstance.messageData.getMessageReadReceipt(
      msgID: message.msgID ?? "",
      userID: message.userID ?? "",
      timestamp: message.timestamp ?? 0,
    );
  }

  @override
  void dispose() {
    _messageDataSubscription?.cancel();
    super.dispose();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    dataProvider = TencentCloudChatMessageDataProviderInherited.of(context);
    _generateMenuOptions();
  }

  void closeTheMenu() {
    setState(() {
      _menuCloser++;
    });
  }

  void _generateMenuOptions({V2TimMessageReceipt? messageReceipt}) {
    final TencentCloudChatMessageConfig config = dataProvider.config;

    final List<TencentCloudChatMessageGeneralOptionItem> additionalOptions = config.additionalMessageMenuOptions(
      userID: dataProvider.userID,
      groupID: dataProvider.groupID,
    );

    // === Group Message Read Receipt ===
    final useReadReceipt = TencentCloudChatUtils.checkString(dataProvider.groupID) != null &&
        TencentCloudChat.dataInstance.basic.messageConfig
            .enabledGroupTypesForMessageReadReceipt(
              userID: dataProvider.userID,
              groupID: dataProvider.groupID,
            )
            .contains(dataProvider.groupInfo?.groupType);
    final showReadReceipt = useReadReceipt && (message.isSelf ?? true) && (message.needReadReceipt ?? false);
    final receipt = showReadReceipt
        ? messageReceipt ??
            TencentCloudChat.dataInstance.messageData.getMessageReadReceipt(
              msgID: message.msgID ?? "",
              userID: message.userID ?? "",
              timestamp: message.timestamp ?? 0,
            )
        : null;
    final int? readCount = showReadReceipt ? receipt?.readCount : null;
    final int? unreadCount = showReadReceipt ? receipt?.unreadCount : null;
    final bool isAllRead = (readCount ?? 0) > 0 && unreadCount == 0;

    final defaultMenuOptions = [
      TencentCloudChatMessageGeneralOptionItem(
          icon: Icons.copy,
          label: tL10n.copy,
          onTap: () {
            closeTheMenu();
            final text = message.textElem?.text ?? "";
            Clipboard.setData(
              ClipboardData(text: text),
            );
          }),
      TencentCloudChatMessageGeneralOptionItem(
          icon: Icons.reply_outlined,
          label: tL10n.reply,
          onTap: () {
            closeTheMenu();
            dataProvider.repliedMessage = message;
          }),
      TencentCloudChatMessageGeneralOptionItem(
        icon: Icons.check_circle_outline,
        label: tL10n.select,
        onTap: () {
          closeTheMenu();
          Future.delayed(
            const Duration(milliseconds: 150),
            () {
              dataProvider.inSelectMode = true;
              dataProvider.selectedMessages = [message];
            },
          );
        },
      ),
      TencentCloudChatMessageGeneralOptionItem(
          icon: Icons.forward,
          label: tL10n.forward,
          onTap: () {
            closeTheMenu();
            showModalBottomSheet<void>(
              context: context,
              isScrollControlled: true,
              shape: const RoundedRectangleBorder(
                borderRadius: BorderRadius.vertical(top: Radius.circular(16)),
              ),
              builder: (BuildContext _) {
                return TencentCloudChatMessageForwardContainer(
                  type: TencentCloudChatForwardType.individually,
                  context: context,
                  messages: [message],
                );
              },
            );
          }),
      TencentCloudChatMessageGeneralOptionItem(
          icon: Icons.delete_outline_outlined,
          label: tL10n.delete,
          onTap: () {
            closeTheMenu();
            showDialog(
              context: context,
              builder: (BuildContext context) {
                final TencentCloudChatMessageConfig config = dataProvider.config;
                final showDeleteForEveryone = (message.isSelf ?? false) && config.enableMessageDeleteForEveryone(userID: dataProvider.userID, groupID: dataProvider.groupID);
                return AlertDialog(
                  title: Text(tL10n.confirmDeletion),
                  content: Text(tL10n.askDeleteThisMessage),
                  actions: [
                    if (showDeleteForEveryone)
                      TextButton(
                        onPressed: () {
                          dataProvider.deleteMessagesForEveryone(messages: [message]);
                          Navigator.pop(context);
                        },
                        child: Text(tL10n.deleteForEveryone),
                      ),
                    TextButton(
                      onPressed: () {
                        dataProvider.deleteMessagesForMe(messages: [message]);
                        Navigator.pop(context);
                      },
                      child: Text(tL10n.deleteForMe),
                    ),
                    TextButton(
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      child: Text(tL10n.cancel),
                    ),
                  ],
                );
              },
            );
          }),
      TencentCloudChatMessageGeneralOptionItem(
          icon: Icons.undo_outlined,
          label: tL10n.recall,
          onTap: () {
            closeTheMenu();
            showDialog(
              context: context,
              builder: (BuildContext context) {
                return AlertDialog(
                  title: Text(tL10n.messageRecall),
                  content: Text(tL10n.messageRecallConfirmation),
                  actions: [
                    TextButton(
                      onPressed: () {
                        dataProvider.recallMessage(message: message);
                        Navigator.pop(context);
                      },
                      child: Text(tL10n.recall),
                    ),
                    TextButton(
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      child: Text(tL10n.cancel),
                    ),
                  ],
                );
              },
            );
          }),
      if (showReadReceipt)
        TencentCloudChatMessageGeneralOptionItem(
            icon: Icons.visibility,
            label: isAllRead ? tL10n.allMembersRead : tL10n.memberReadCount(readCount ?? 0),
            onTap: () {
              closeTheMenu();
            }),
    ];

    final mergedList = [...defaultMenuOptions, ...additionalOptions];

    if (message.elemType != MessageElemType.V2TIM_ELEM_TYPE_TEXT) {
      mergedList.removeWhere((element) => element.label == tL10n.copy);
    }
    if (!(message.isSelf ?? false)) {
      mergedList.removeWhere((element) => element.label == tL10n.recall);
    }

    setState(() {
      _menuOptions = mergedList;
    });
  }

  @override
  Widget defaultBuilder(BuildContext context) {
    return TencentCloudChatMessageBuilders.getMessageLongPressBuilder(
      messageItem: widget.messageItem,
      useMessageReaction: widget.useMessageReaction,
      message: message,
      menuOptions: _menuOptions,
      menuCloser: _menuCloser,
      isMergeMessage: widget.isMergeMessage,
      inSelectMode: dataProvider.inSelectMode,
      onSelectMessage: () => dataProvider.triggerSelectedMessage(message: message),
    );
  }
}
